import logger from './logger.mjs'
import { sendWebhook } from './notifications.mjs'


export async function main(browser, scenario) {
  await runScenario(scenario, browser)
}

async function runScenario(scenario, browser) {
  for (let pageScenario of scenario.pages) {
    let page = await browser.newPage()

    await runPageScenario(pageScenario, page)

    await page.close()
  }
}

async function runPageScenario(pageScenario, page) {
  try {
    if (Array.isArray(pageScenario.url)) {
      for (const url of pageScenario.url) {
        await runUrl(url, pageScenario, page)
      }
    } else if (typeof pageScenario.url === 'string') {
      await runUrl(pageScenario.url, pageScenario, page)
    } else {
      throw Error('url should be either string or string[]')
    }
  } catch (err) {
    logger.error(err)
  }
}

async function runUrl(url, pageScenario, page) {
  await page.goto(url, {
    waitUntil: 'domcontentloaded',
  })

  if (pageScenario.waitForSelector) {
    await page.waitForSelector(pageScenario.waitForSelector)
  }

  if (pageScenario.waitForXPath) {
    await page.waitForXPath(pageScenario.waitForXPath)
  }

  const output = await runSchema(pageScenario.schema, page)

  if (pageScenario.notifications) {
    for (const channel of pageScenario.notifications) {
      await informPageResult(channel, output)
    }
  }
}

async function runSchema(schema, handle) {
  switch (schema.type) {
    case 'array': {
      let children = []

      if (schema.selector) {
        logger.debug(`Searching for: ${schema.selector}`)
        children = await handle.$$(schema.selector)
      } else if (schema.xpath) {
        logger.debug(`Searching for: ${schema.xpath}`)
        children = await handle.$x(schema.xpath)
      } else {
        throw new Error('Missing selector or xpath')
      }


      const output = []

      for (const child of children) {
        output.push(await runSchema(schema.items, child))
      }

      return output
    }
    case 'object': {
      const output = {}

      for (const property in schema.properties) {
        output[property] = await runSchema(schema.properties[property], handle)
      }

      return output
    }
    case 'string': {
      if (schema.selector) {
        logger.debug(`Searching for: ${schema.selector}`)
        return await handle.$eval(schema.selector, node => node.textContent)
      } else if (schema.xpath) {
        logger.debug(`Searching for: ${schema.xpath}`)
        const [child] = await handle.$x(schema.xpath)
        return await child.evaluate(node => node.textContent)
      } else {
        throw new Error('Missing selector or xpath')
      }
    }
    default:
      throw new Error('Unsupported schema type')
  }
}

async function informPageResult(channel, data) {
  if (channel.type === 'webhook') {
    return await sendWebhook(channel.url, data)
  } else if (channel.type === 'console') {
    logger.info(JSON.stringify(data))
    return
  } else {
    throw new Error('Unsupported notification channel')
  }
}